@@ -7,87 +7,29 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::{QueryResult, Session};
+
 use crate::client::Error;
 use crate::messaging::{
     data::{ChunkRead, DataQuery, QueryResponse},
-    section_info::SectionInfoMsg,
-    MessageId, ServiceAuth, WireMsg,
+    DstLocation, MessageId, MsgKind, ServiceAuth, WireMsg,
 };
-use crate::messaging::{DstLocation, MsgKind};
-use crate::types::{Chunk, PrivateChunk, PublicChunk, PublicKey};
+use crate::types::{Chunk, PrivateChunk, PublicChunk};
+
 use bytes::Bytes;
 use futures::{future::join_all, stream::FuturesUnordered};
 use itertools::Itertools;
-use rand::prelude::SliceRandom;
-use rand::rngs::OsRng;
-use std::{collections::BTreeSet, net::SocketAddr, time::Duration};
-use tokio::{sync::mpsc::channel, task::JoinHandle, time::timeout};
+use qp2p::Endpoint;
+use std::{collections::BTreeMap, net::SocketAddr};
+use tokio::{sync::mpsc::channel, task::JoinHandle, time::sleep};
 use tracing::{debug, error, trace, warn};
 use xor_name::XorName;
 
-// Number of attemps when retrying to send a message to a node
+// Number of attempts when retrying to send a message to a node
 const NUMBER_OF_RETRIES: usize = 3;
 // Number of Elders subset to send queries to
 const NUM_OF_ELDERS_SUBSET_FOR_QUERIES: usize = 3;
 
 impl Session {
-    /// Bootstrap to the network maintaining connections to several nodes.
-    pub(crate) async fn bootstrap(&mut self, client_pk: PublicKey) -> Result<(), Error> {
-        trace!(
-            "Trying to bootstrap to the network with public_key: {:?}",
-            client_pk
-        );
-
-        let (endpoint, _, mut incoming_messages, _disconnections, mut bootstrapped_peer) =
-            self.qp2p.bootstrap().await?;
-
-        self.endpoint = Some(endpoint.clone());
-        let mut bootstrap_nodes = endpoint
-            .clone()
-            .bootstrap_nodes()
-            .to_vec()
-            .into_iter()
-            .collect::<BTreeSet<_>>();
-
-        self.send_get_section_query(client_pk, &bootstrapped_peer)
-            .await?;
-
-        // Bootstrap and send a handshake request to the bootstrapped peer
-        let mut we_have_keyset = false;
-        while !we_have_keyset {
-            // This means that the peer we bootstrapped to
-            // has responded with a SectionInfo Message
-            if let Ok(Ok(true)) = timeout(
-                Duration::from_secs(30),
-                self.process_incoming_message(&mut incoming_messages, client_pk),
-            )
-            .await
-            {
-                we_have_keyset = self.section_key_set.read().await.is_some();
-            } else {
-                // Remove the unresponsive peer we boostrapped to and bootstrap again
-                let _ = bootstrap_nodes.remove(&bootstrapped_peer);
-                bootstrapped_peer = self
-                    .qp2p
-                    .rebootstrap(
-                        &endpoint,
-                        &bootstrap_nodes.iter().cloned().collect::<Vec<_>>(),
-                    )
-                    .await?;
-            }
-        }
-
-        self.spawn_message_listener_thread(incoming_messages, client_pk)
-            .await;
-
-        debug!(
-            "Successfully obtained the list of Elders to send all messages to: {:?}",
-            self.connected_elders.read().await.keys()
-        );
-
-        Ok(())
-    }
-
     /// Send a `ServiceMsg` to the network without awaiting for a response.
     pub(crate) async fn send_cmd(
         &self,
@@ -98,41 +40,41 @@ impl Session {
     ) -> Result<(), Error> {
         let endpoint = self.endpoint()?.clone();
 
-        let mut elders = self
-            .connected_elders
-            .read()
-            .await
-            .keys()
-            .cloned()
-            .collect::<Vec<SocketAddr>>();
-
-        if targets < elders.len() {
-            elders.shuffle(&mut OsRng);
-        }
-
-        elders = elders.into_iter().take(targets).collect();
+        // Get DataSection elders details.
+        let (elders, section_pk) = if let Some(sap) = self.network.closest_or_opposite(&dst_address)
+        {
+            (
+                sap.value
+                    .elders
+                    .values()
+                    .cloned()
+                    .take(targets)
+                    .collect::<Vec<SocketAddr>>(),
+                sap.value.public_key_set.public_key(),
+            )
+        } else {
+            // Send message to our bootstrap peer with network's genesis PK.
+            self.bootstrap_peer
+                .map(|addr| (vec![addr], self.genesis_pk))
+                .ok_or(Error::NotBootstrapped)?
+        };
 
         let msg_id = MessageId::new();
+
         debug!(
             "Sending command w/id {:?}, from {}, to {} Elders",
             msg_id,
-            endpoint.socket_addr(),
+            endpoint.public_addr(),
             elders.len()
         );
 
         trace!(
             "Sending (from {}) dst {:?} w/ id: {:?}",
-            endpoint.socket_addr(),
+            endpoint.public_addr(),
             dst_address,
             msg_id
         );
 
-        let section_pk = self
-            .section_key()
-            .await?
-            .bls()
-            .ok_or(Error::NoBlsSectionKey)?;
-
         let dst_location = DstLocation::Section {
             name: dst_address,
             section_pk,
@@ -140,41 +82,7 @@ impl Session {
         let msg_kind = MsgKind::ServiceMsg(auth);
         let wire_msg = WireMsg::new_msg(msg_id, payload, msg_kind, dst_location)?;
 
-        let msg_bytes = wire_msg.serialize()?;
-
-        // Send message to all Elders concurrently
-        let mut tasks = Vec::default();
-
-        // clone elders as we want to update them in this process
-        for socket in elders {
-            let msg_bytes_clone = msg_bytes.clone();
-            let endpoint = endpoint.clone();
-            let task_handle: JoinHandle<Result<(), Error>> = tokio::spawn(async move {
-                trace!("About to send cmd message {:?} to {:?}", msg_id, &socket);
-                endpoint.connect_to(&socket).await?;
-                endpoint.send_message(msg_bytes_clone, &socket).await?;
-
-                trace!("Sent cmd with MsgId {:?}to {:?}", msg_id, &socket);
-                Ok(())
-            });
-            tasks.push(task_handle);
-        }
-
-        // Let's await for all messages to be sent
-        let results = join_all(tasks).await;
-
-        let mut failures = 0;
-        results.iter().for_each(|res| {
-            if res.is_err() {
-                failures += 1;
-            }
-        });
-
-        if failures > 0 {
-            error!("Sending the message to {} Elders failed", failures);
-        }
-
-        Ok(())
+        send_message(elders, wire_msg, self.endpoint.clone(), msg_id).await
     }
 
     /// Send a `ServiceMsg` to the network awaiting for the response.
@@ -183,6 +91,7 @@ impl Session {
         query: DataQuery,
         auth: ServiceAuth,
         payload: Bytes,
+        msg_id: MessageId,
     ) -> Result<QueryResult, Error> {
         let endpoint = self.endpoint()?.clone();
         let pending_queries = self.pending_queries.clone();
@@ -193,29 +102,44 @@ impl Session {
             None
         };
 
-        let section_pk = self
-            .section_key()
-            .await?
-            .bls()
-            .ok_or(Error::NoBlsSectionKey)?;
+        let data_name = query.dst_name();
 
-        let data_name = query.dst_address();
+        // Get DataSection elders details. Resort to own section if DataSection is not available.
+        let (elders, section_pk) = if let Some(sap) = self.network.closest_or_opposite(&data_name) {
+            (sap.value.elders, sap.value.public_key_set.public_key())
+        } else {
+            // Send message to our bootstrap peer with the network's genesis PK and addressing adring.
+            self.bootstrap_peer
+                .map(|addr| {
+                    let mut bootstrapped_peer = BTreeMap::new();
+                    let _ = bootstrapped_peer.insert(XorName::random(), addr);
+                    (bootstrapped_peer, self.genesis_pk)
+                })
+                .ok_or(Error::NotBootstrapped)?
+        };
 
-        // We select the NUM_OF_ELDERS_SUBSET_FOR_QUERIES closest
-        // connected Elders to the data we are querying
-        let elders: Vec<SocketAddr> = self
-            .connected_elders
-            .read()
-            .await
-            .clone()
+        // We select the NUM_OF_ELDERS_SUBSET_FOR_QUERIES closest Elders we are querying
+        let chosen_elders = elders
             .into_iter()
-            .sorted_by(|(_, lhs_name), (_, rhs_name)| data_name.cmp_distance(lhs_name, rhs_name))
+            .sorted_by(|(lhs_name, _), (rhs_name, _)| data_name.cmp_distance(lhs_name, rhs_name))
+            .map(|(_, addr)| addr)
             .take(NUM_OF_ELDERS_SUBSET_FOR_QUERIES)
-            .map(|(addr, _)| addr)
-            .collect();
+            .collect::<Vec<SocketAddr>>();
 
-        let elders_len = elders.len();
-        if elders_len < NUM_OF_ELDERS_SUBSET_FOR_QUERIES {
+        let wire_msg = WireMsg::new_msg(
+            msg_id,
+            payload,
+            MsgKind::ServiceMsg(auth),
+            DstLocation::Section {
+                name: data_name,
+                section_pk,
+            },
+        )?;
+        let priority = wire_msg.msg_kind().priority();
+        let msg_bytes = wire_msg.serialize()?;
+
+        let elders_len = chosen_elders.len();
+        if elders_len < NUM_OF_ELDERS_SUBSET_FOR_QUERIES && elders_len > 1 {
             error!(
                 "Not enough Elder connections: {}, minimum required: {}",
                 elders_len, NUM_OF_ELDERS_SUBSET_FOR_QUERIES
@@ -229,9 +153,9 @@ impl Session {
             "Sending query message {:?}, msg_id: {}, from {}, to the {} Elders closest to data name: {:?}",
             query,
             msg_id,
-            endpoint.socket_addr(),
+            endpoint.public_addr(),
             elders_len,
-            elders
+            chosen_elders
         );
 
         // We send the same message to all Elders concurrently
@@ -239,39 +163,34 @@ impl Session {
         let (sender, mut receiver) = channel::<QueryResponse>(7);
 
         let pending_queries_for_thread = pending_queries.clone();
-        let _ = tokio::spawn(async move {
-            // Remove the response sender
-            trace!("Inserting channel for {:?}", msg_id);
-            let _ = pending_queries_for_thread
-                .write()
-                .await
-                .insert(msg_id, sender);
-        });
+        if let Ok(op_id) = query.operation_id() {
+            let _ = tokio::spawn(async move {
+                // Insert the response sender
+                trace!("Inserting channel for {:?}", op_id);
+                let _ = pending_queries_for_thread
+                    .write()
+                    .await
+                    .insert(op_id, sender);
+            });
+        }
 
         let discarded_responses = std::sync::Arc::new(tokio::sync::Mutex::new(0_usize));
 
-        let dst_location = DstLocation::Section {
-            name: data_name,
-            section_pk,
-        };
-        let msg_kind = MsgKind::ServiceMsg(auth);
-        let wire_msg = WireMsg::new_msg(msg_id, payload, msg_kind, dst_location)?;
-        let msg_bytes = wire_msg.serialize()?;
-
         // Set up response listeners
-        for socket in elders {
+        for socket in chosen_elders {
             let endpoint = endpoint.clone();
             let msg_bytes = msg_bytes.clone();
             let counter_clone = discarded_responses.clone();
             let task_handle = tokio::spawn(async move {
-                endpoint.connect_to(&socket).await?;
-
                 // Retry queries that failed due to connection issues only
                 let mut result = Err(Error::ElderQuery);
                 for attempt in 0..NUMBER_OF_RETRIES + 1 {
                     let msg_bytes_clone = msg_bytes.clone();
 
-                    if let Err(err) = endpoint.send_message(msg_bytes_clone, &socket).await {
+                    if let Err(err) = endpoint
+                        .send_message(msg_bytes_clone, &socket, priority)
+                        .await
+                    {
                         error!(
                             "Try #{:?} @ {:?}, failed sending query message: {:?}",
                             attempt + 1,
@@ -281,7 +200,7 @@ impl Session {
                         result = Err(Error::SendingQuery);
                         if attempt <= NUMBER_OF_RETRIES {
                             let millis = 2_u64.pow(attempt as u32 - 1) * 100;
-                            tokio::time::sleep(std::time::Duration::from_millis(millis)).await
+                            sleep(std::time::Duration::from_millis(millis)).await
                         }
                     } else {
                         trace!("ServiceMsg with id: {:?}, sent to {}", &msg_id, &socket);
@@ -330,12 +249,12 @@ impl Session {
         let response = loop {
             let mut error_response = None;
             match (receiver.recv().await, chunk_addr) {
-                (Some(QueryResponse::GetChunk(Ok(blob))), Some(chunk_addr)) => {
+                (Some(QueryResponse::GetChunk(Ok(chunk))), Some(chunk_addr)) => {
                     // We are dealing with Chunk query responses, thus we validate its hash
                     // matches its xorname, if so, we don't need to await for more responses
-                    debug!("Chunk QueryResponse received is: {:#?}", blob);
+                    debug!("Chunk QueryResponse received is: {:#?}", chunk);
 
-                    let xorname = match &blob {
+                    let xorname = match &chunk {
                         Chunk::Private(priv_chunk) => {
                             *PrivateChunk::new(priv_chunk.value().clone(), *priv_chunk.owner())
                                 .name()
@@ -347,7 +266,7 @@ impl Session {
 
                     if *chunk_addr.name() == xorname {
                         trace!("Valid Chunk received for {}", msg_id);
-                        break Some(QueryResponse::GetChunk(Ok(blob)));
+                        break Some(QueryResponse::GetChunk(Ok(chunk)));
                     } else {
                         // the Chunk content doesn't match its Xorname,
                         // this is suspicious and it could be a byzantine node
@@ -357,12 +276,13 @@ impl Session {
                 }
                 // Erring on the side of positivity. \
                 // Saving error, but not returning until we have more responses in
-                // (note, this will overwrite prior errors, so we'll just return whicever was last received)
+                // (note, this will overwrite prior errors, so we'll just return whichever was last received)
                 (response @ Some(QueryResponse::GetChunk(Err(_))), Some(_))
-                | (response @ Some(QueryResponse::GetRegister(Err(_))), None)
-                | (response @ Some(QueryResponse::GetRegisterPolicy(Err(_))), None)
-                | (response @ Some(QueryResponse::GetRegisterOwner(Err(_))), None)
-                | (response @ Some(QueryResponse::GetRegisterUserPermissions(Err(_))), None) => {
+                | (response @ Some(QueryResponse::GetRegister((Err(_), _))), None)
+                | (response @ Some(QueryResponse::GetRegisterPolicy((Err(_), _))), None)
+                | (response @ Some(QueryResponse::GetRegisterOwner((Err(_), _))), None)
+                | (response @ Some(QueryResponse::GetRegisterUserPermissions((Err(_), _))), None) =>
+                {
                     debug!("QueryResponse error received (but may be overridden by a non-error reponse from another elder): {:#?}", &response);
                     error_response = response;
                     discarded_responses += 1;
@@ -386,96 +306,85 @@ impl Session {
             msg_id, response
         );
 
-        let _ = tokio::spawn(async move {
-            // Remove the response sender
-            trace!("Removing channel for {:?}", msg_id);
-            let _ = pending_queries.clone().write().await.remove(&msg_id);
-        });
-
-        response
-            .map(|response| QueryResult { response, msg_id })
-            .ok_or(Error::NoResponse)
-    }
-
-    // Get section info from the peer we have bootstrapped with.
-    pub(crate) async fn send_get_section_query(
-        &self,
-        client_pk: PublicKey,
-        bootstrapped_peer: &SocketAddr,
-    ) -> Result<(), Error> {
-        if self.is_connecting_to_new_elders {
-            // This should ideally be unreachable code. Leaving it while this is a WIP
-            error!("Already attempting elder connections, not sending section query until that is complete.");
-            return Ok(());
+        if let Some(query) = &response {
+            if let Ok(query_op_id) = query.operation_id() {
+                let _ = tokio::spawn(async move {
+                    // Remove the response sender
+                    trace!("Removing channel for {:?}", query_op_id);
+                    let _ = pending_queries.clone().write().await.remove(&query_op_id);
+                });
+            }
         }
 
-        trace!(
-            "Querying for section info from bootstrapped node: {:?}",
-            bootstrapped_peer
-        );
-
-        let dst_section_name = XorName::from(client_pk);
-
-        // FIXME: we don't know our section PK. We must supply a pk for now we do a random one...
-        let random_section_pk = bls::SecretKey::random().public_key();
-
-        let msg_id = MessageId::new();
-        let query = SectionInfoMsg::GetSectionQuery(client_pk);
-        let payload = WireMsg::serialize_msg_payload(&query)?;
-        let msg_kind = MsgKind::SectionInfoMsg;
-        let dst_location = DstLocation::Section {
-            name: dst_section_name,
-            section_pk: random_section_pk,
-        };
-        let wire_msg = WireMsg::new_msg(msg_id, payload, msg_kind, dst_location)?;
-        let msg_bytes = wire_msg.serialize()?;
-
-        debug!(
-            "Sending query message {:?}, msg_id {}, from {}, to bootstrapped node {} ",
-            query,
-            msg_id,
-            self.endpoint()?.socket_addr(),
-            bootstrapped_peer
-        );
-
-        self.endpoint()?
-            .send_message(msg_bytes, bootstrapped_peer)
-            .await?;
-
-        Ok(())
+        match response {
+            Some(response) => {
+                let operation_id = response
+                    .operation_id()
+                    .map_err(|_| Error::UnknownOperationId)?;
+                Ok(QueryResult {
+                    response,
+                    operation_id,
+                })
+            }
+            None => Err(Error::NoResponse),
+        }
     }
 
+    #[allow(unused)]
     pub(crate) async fn disconnect_from_peers(&self, peers: Vec<SocketAddr>) -> Result<(), Error> {
         for elder in peers {
-            self.endpoint()?.disconnect_from(&elder).await?;
+            self.endpoint()?.disconnect_from(&elder).await;
         }
 
         Ok(())
     }
+}
 
-    // Connect to a set of Elders nodes which will be
-    // the receipients of our messages on the network.
-    pub(crate) async fn connect_to_elders(&mut self) -> Result<(), Error> {
-        // TODO: remove this function completely
+pub(crate) async fn send_message(
+    elders: Vec<SocketAddr>,
+    wire_msg: WireMsg,
+    endpoint: Option<Endpoint<XorName>>,
+    msg_id: MessageId,
+) -> Result<(), Error> {
+    let priority = wire_msg.msg_kind().priority();
+    let msg_bytes = wire_msg.serialize()?;
+    let endpoint = match endpoint {
+        Some(ep) => ep,
+        None => return Err(Error::NotBootstrapped),
+    };
 
-        self.is_connecting_to_new_elders = true;
+    // Send message to all Elders concurrently
+    let mut tasks = Vec::default();
 
-        if self.known_elders_count().await == 0 {
-            // this is not necessarily an error in case we didn't get elder info back yet
-            warn!("Not attempted to connect, insufficient elders yet known");
-        }
+    // clone elders as we want to update them in this process
+    for socket in elders {
+        let msg_bytes_clone = msg_bytes.clone();
+        let endpoint = endpoint.clone();
+        let task_handle: JoinHandle<Result<(), Error>> = tokio::spawn(async move {
+            trace!("About to send cmd message {:?} to {:?}", msg_id, &socket);
+            endpoint
+                .send_message(msg_bytes_clone, &socket, priority)
+                .await?;
+
+            trace!("Sent cmd with MsgId {:?} to {:?}", msg_id, &socket);
+            Ok(())
+        });
+        tasks.push(task_handle);
+    }
 
-        let new_elders = self.all_known_elders.read().await.clone();
-        let peers_len = new_elders.len();
+    // Let's await for all messages to be sent
+    let results = join_all(tasks).await;
 
-        trace!("We now know our {} Elders.", peers_len);
-        {
-            let mut session_elders = self.connected_elders.write().await;
-            *session_elders = new_elders;
+    let mut failures = 0;
+    results.iter().for_each(|res| {
+        if res.is_err() {
+            failures += 1;
         }
+    });
 
-        self.is_connecting_to_new_elders = false;
+    if failures > 0 {
+        error!("Sending the message to {} Elders failed", failures);
+    }
 
-        Ok(())
-    }
+    Ok(())
 }
