@@ -9,13 +9,12 @@
 use std::cmp;
 
 use crate::messaging::{
-    node::{KeyedSig, MembershipState, NodeMsg, NodeState, Proposal, SectionAuth},
+    system::{KeyedSig, MembershipState, NodeState, Proposal, SectionAuth},
     SectionAuthorityProvider,
 };
 use crate::routing::{
     dkg::SectionAuthUtils,
     error::Result,
-    network::NetworkUtils,
     peer::PeerUtils,
     routing_api::command::Command,
     section::{ElderCandidatesUtils, SectionPeersUtils, SectionUtils},
@@ -108,7 +107,7 @@ impl Core {
 
         let result = self.promote_and_demote_elders()?;
         if result.is_empty() {
-            commands.extend(self.send_sync_to_adults()?);
+            commands.extend(self.send_ae_update_to_adults()?);
         }
 
         commands.extend(result);
@@ -143,7 +142,7 @@ impl Core {
 
         let result = self.promote_and_demote_elders()?;
         if result.is_empty() {
-            commands.extend(self.send_sync_to_adults()?);
+            commands.extend(self.send_ae_update_to_adults()?);
         }
 
         commands.extend(result);
@@ -162,35 +161,35 @@ impl Core {
         section_auth: SectionAuthorityProvider,
         sig: KeyedSig,
     ) -> Result<Vec<Command>> {
-        let mut commands = vec![];
         let equal_or_extension = section_auth.prefix() == *self.section.prefix()
             || section_auth.prefix().is_extension_of(self.section.prefix());
-        let signed_section_auth = SectionAuth::new(section_auth, sig.clone());
 
         if equal_or_extension {
-            // Our section of sub-section
-
+            // Our section or sub-section
+            let signed_section_auth = SectionAuth::new(section_auth, sig.clone());
             let infos = self.section.promote_and_demote_elders(&self.node.name());
             if !infos.contains(&signed_section_auth.value.elder_candidates()) {
                 // SectionInfo out of date, ignore.
-                return Ok(commands);
+                return Ok(vec![]);
             }
 
-            // Send a `Sync` message to all the to-be-promoted members so they have the full
+            // Send a `AE Update` message to all the to-be-promoted members so they have the full
             // section and network data.
-            let sync_recipients: Vec<_> = infos
+            let ae_update_recipients: Vec<_> = infos
                 .iter()
                 .flat_map(|info| info.peers())
                 .filter(|peer| !self.section.is_elder(peer.name()))
                 .map(|peer| (*peer.name(), *peer.addr()))
                 .collect();
-            if !sync_recipients.is_empty() {
-                let node_msg = NodeMsg::Sync {
-                    section: self.section.clone(),
-                    network: self.network.clone(),
-                };
-                let cmd =
-                    self.send_direct_message_to_nodes(sync_recipients, node_msg, sig.public_key)?;
+
+            let mut commands = vec![];
+            if !ae_update_recipients.is_empty() {
+                let node_msg = self.generate_ae_update(sig.public_key, true)?;
+                let cmd = self.send_direct_message_to_nodes(
+                    ae_update_recipients,
+                    node_msg,
+                    sig.public_key,
+                )?;
 
                 commands.push(cmd);
             }
@@ -202,18 +201,17 @@ impl Core {
                 &our_elders_recipients,
                 Proposal::OurElders(signed_section_auth),
             )?);
-        } else {
-            // Other section
-            // FIXME: we shouln't be receiving or updating a SAP for
-            // a remote section here, that would be done with a AE msg response ???.
-            let _ = self.network.update_remote_section_sap(
-                signed_section_auth,
-                self.section.chain(),
-                self.section.chain(),
-            );
-        }
 
-        Ok(commands)
+            Ok(commands)
+        } else {
+            // Other section. We shouln't be receiving or updating a SAP for
+            // a remote section here, that is done with a AE msg response.
+            debug!(
+                "Ignoring Proposal::SectionInfo since prefix doesn't match ours: {:?}",
+                section_auth
+            );
+            Ok(vec![])
+        }
     }
 
     async fn handle_our_elders_agreement(
@@ -223,27 +221,40 @@ impl Core {
     ) -> Result<Vec<Command>> {
         let updates =
             self.split_barrier
-                .process(self.section.prefix(), signed_section_auth, key_sig);
+                .process(self.section.prefix(), signed_section_auth.clone(), key_sig);
         if updates.is_empty() {
             return Ok(vec![]);
         }
 
         let snapshot = self.state_snapshot();
+        let mut old_chain = self.section.chain.clone();
 
         for (section_auth, key_sig) in updates {
+            info!("Updating {:?}", &section_auth);
             if section_auth.value.prefix.matches(&self.node.name()) {
-                let _ = self.section.update_elders(section_auth, key_sig);
+                let _ = self.section.update_elders(section_auth.clone(), key_sig);
+                if self.network.update(section_auth, self.section_chain())? {
+                    info!("Updated our section's state in network's NetworkPrefixMap");
+                }
             } else {
-                // FIXME: we shouln't be receiving or updating a SAP for
-                // a remote section here, that should be done with a AE msg response.
-                let _ = self.network.update_remote_section_sap(
-                    section_auth,
-                    self.section.chain(),
-                    self.section.chain(),
-                );
+                // Update the old chain to become the neighbour's chain.
+                if let Err(e) = old_chain.insert(
+                    &key_sig.public_key,
+                    section_auth.value.section_key(),
+                    key_sig.signature,
+                ) {
+                    error!("Error generating neighbouring section's proof_chain for knowledge update on split: {:?}", e);
+                }
+
+                info!("Updating neighbouring section's SAP");
+                if let Err(e) = self.network.update(section_auth, &old_chain) {
+                    error!("Error updating neighbouring section's details on our NetworkPrefixMap: {:?}", e);
+                }
             }
         }
 
+        info!("Prefixes we know about: {:?}", self.network);
+
         self.update_state(snapshot).await
     }
 }
