@@ -9,17 +9,16 @@
 //! Relocation related types and utilities.
 
 use crate::messaging::{
-    node::{
-        Network, NodeMsg, NodeState, Peer, RelocateDetails, RelocatePayload, RelocatePromise,
-        Section,
+    system::{
+        NodeState, Peer, RelocateDetails, RelocatePayload, RelocatePromise, Section, SystemMsg,
     },
     AuthorityProof, SectionAuth,
 };
+use crate::prefix_map::NetworkPrefixMap;
 use crate::routing::{
     core::JoiningAsRelocated,
     ed25519::{self, Keypair, Verifier},
     error::Error,
-    network::NetworkUtils,
     peer::PeerUtils,
     section::{
         section_authority_provider::SectionAuthorityProviderUtils, SectionPeersUtils, SectionUtils,
@@ -30,7 +29,7 @@ use xor_name::XorName;
 /// Find all nodes to relocate after a churn event and create the relocate actions for them.
 pub(crate) fn actions(
     section: &Section,
-    network: &Network,
+    network: &NetworkPrefixMap,
     churn_name: &XorName,
     churn_signature: &bls::Signature,
 ) -> Vec<(NodeState, RelocateAction)> {
@@ -63,11 +62,11 @@ pub(crate) fn actions(
 /// Details of a relocation: which node to relocate, where to relocate it to and what age it should
 /// get once relocated.
 pub(super) trait RelocateDetailsUtils {
-    fn new(section: &Section, network: &Network, peer: &Peer, dst: XorName) -> Self;
+    fn new(section: &Section, network: &NetworkPrefixMap, peer: &Peer, dst: XorName) -> Self;
 
     fn with_age(
         section: &Section,
-        network: &Network,
+        network: &NetworkPrefixMap,
         peer: &Peer,
         dst: XorName,
         age: u8,
@@ -75,13 +74,13 @@ pub(super) trait RelocateDetailsUtils {
 }
 
 impl RelocateDetailsUtils for RelocateDetails {
-    fn new(section: &Section, network: &Network, peer: &Peer, dst: XorName) -> Self {
+    fn new(section: &Section, network: &NetworkPrefixMap, peer: &Peer, dst: XorName) -> Self {
         Self::with_age(section, network, peer, dst, peer.age().saturating_add(1))
     }
 
     fn with_age(
         section: &Section,
-        network: &Network,
+        network: &NetworkPrefixMap,
         peer: &Peer,
         dst: XorName,
         age: u8,
@@ -102,7 +101,7 @@ impl RelocateDetailsUtils for RelocateDetails {
 
 pub(super) trait RelocatePayloadUtils {
     fn new(
-        details: NodeMsg,
+        details: SystemMsg,
         section_auth: AuthorityProof<SectionAuth>,
         new_name: &XorName,
         old_keypair: &Keypair,
@@ -115,7 +114,7 @@ pub(super) trait RelocatePayloadUtils {
 
 impl RelocatePayloadUtils for RelocatePayload {
     fn new(
-        details: NodeMsg,
+        details: SystemMsg,
         section_auth: AuthorityProof<SectionAuth>,
         new_name: &XorName,
         old_keypair: &Keypair,
@@ -148,7 +147,7 @@ impl RelocatePayloadUtils for RelocatePayload {
     }
 
     fn relocate_details(&self) -> Result<&RelocateDetails, Error> {
-        if let NodeMsg::Relocate(relocate_details) = &self.details {
+        if let SystemMsg::Relocate(relocate_details) = &self.details {
             Ok(relocate_details)
         } else {
             error!("RelocateDetails does not contain a NodeMsg::Relocate");
@@ -162,7 +161,7 @@ pub(crate) enum RelocateState {
     // while being an elder. It must keep fulfilling its duties as elder until its demoted, then it
     // can send the bytes (which are serialized `RelocatePromise` message) back to the elders who
     // will exchange it for an actual `Relocate` message.
-    Delayed(NodeMsg),
+    Delayed(SystemMsg),
     // Relocation in progress.
     InProgress(Box<JoiningAsRelocated>),
 }
@@ -179,7 +178,7 @@ pub(crate) enum RelocateAction {
 impl RelocateAction {
     pub(crate) fn new(
         section: &Section,
-        network: &Network,
+        network: &NetworkPrefixMap,
         peer: &Peer,
         churn_name: &XorName,
     ) -> Self {
@@ -222,7 +221,7 @@ pub(crate) fn check(age: u8, churn_signature: &bls::Signature) -> bool {
 // Compute the destination for the node with `relocating_name` to be relocated to. `churn_name` is
 // the name of the joined/left node that triggered the relocation.
 fn dst(relocating_name: &XorName, churn_name: &XorName) -> XorName {
-    XorName::from_content(&[&relocating_name.0, &churn_name.0])
+    XorName::from_content_parts(&[&relocating_name.0, &churn_name.0])
 }
 
 // Returns the number of trailing zero bits of the byte slice.
@@ -292,7 +291,7 @@ mod tests {
 
         let sk_set = SecretKeySet::random();
         let sk = sk_set.secret_key();
-        let pk = sk.public_key();
+        let genesis_pk = sk.public_key();
 
         // Create `Section` with `peers` as its members and set the `ELDER_SIZE` oldest peers as
         // the elders.
@@ -308,7 +307,8 @@ mod tests {
         );
         let section_auth = section_signed(sk, section_auth)?;
 
-        let mut section = Section::new(pk, SecuredLinkedList::new(pk), section_auth)?;
+        let mut section =
+            Section::new(genesis_pk, SecuredLinkedList::new(genesis_pk), section_auth)?;
 
         for peer in &peers {
             let info = NodeState::joined(*peer, None);
@@ -317,7 +317,7 @@ mod tests {
             assert!(section.update_member(info));
         }
 
-        let network = Network::new();
+        let network = NetworkPrefixMap::new(genesis_pk);
 
         // Simulate a churn event whose signature has the given number of trailing zeros.
         let churn_name = rng.gen();
